<?php
/*
Contains all functionality for Shodown.

Author: Remco Kouw
Site: http://www.hacksuite.com
Last Edit: 01-02-2019
*/

/* Shodown Class */
class Shodown{
	var $iMatches = 10;
	var $iPage = -1;
	var $iMethod = -1;
	/* main */
	function Shodown(){
	}
	/* getter and setter methods */
	function GetMatches(){
		return($this->iMatches);
	}
	function SetMatches($iMatches){
		$this->iMatches = $iMatches;
	}
	function GetPage(){
		return($this->iPage);
	}
	function SetPage($iPage){
		$this->iPage = $iPage;
	}
	function GetMethod(){
		return($this->iMethod);
	}
	function SetMethod($iMethod){
		$this->iMethod = $iMethod;
	}
	function GetSearch(){
		return($this->sSearch);
	}
	function SetSearch($sSearch){
		$this->sSearch = $sSearch;
	}
	/* fetches all filenames from a directory */
	function GetFilesByDirectory($sDir){
		if(!$rHandle = @opendir($sDir)){
			return(false);
		}
		$aFileBuffer = array();
		while(false!==($sFile = @readdir($rHandle))){
			// buffer all files
			if($sFile!="." && $sFile!=".."){
				if(!is_dir($sDir."/".$sFile)){
					$aFileBuffer[] .= $sFile;
				}
			}
		}
		@closedir($rHandle);
		return $aFileBuffer;
	}
	/* writes data to a file, overwrites it or creates a new file with the content in $sWrite */
	function WriteF($sDest,$sWrite,$sMode="a"){
		if($sMode!="a" && $sMode!="w"){
			$sMode = "a";
		}
		if(!$rNew = @fopen($sDest,$sMode)){
			return(false);
		}
		@fputs($rNew,$sWrite);
		@fclose($rNew);
		return(true);
	}
	/* creates a link to a BugTraq page */
	function BugTraqID($iID){
		return("<a href=\"https://www.securityfocus.com/bid/".$iID."\" target=\"blank\">".$iID."</a>");
	}
	/* creates a link to a CVE page */
	function CVEID($iID){
		return("<a href=\"https://nvd.nist.gov/vuln/detail/".$iID."\" target=\"blank\">".$iID."</a>");
	}
	/* creates a link to a OSVDB page */
	function OSVDB($iID){
		return("<a href=\"https://vulners.com/osvdb/OSVDB:".$iID."\" target=\"blank\">".$iID."</a>");
	}
	/* creates a link to an ASN lookup */
	function ASN($sID){
		return("<a href=\"https://api.hackertarget.com/aslookup/?q=".$sID."\" target=\"blank\">".$sID."</a>");
	}
	/* creates a link to a Google */
	function Google($sSearch){
		return("<a href=\"http://www.google.com/search?q=".urlencode($sSearch)."\" target=\"blank\">".$sSearch."</a>");
	}
	/* creates a link to a Google Feeling Lucky(1st search result) */
	function GoogleFeelingLucky($sSearch){
		return("<a href=\"http://www.google.com/search?q=".urlencode($sSearch)."&btnI\" target=\"blank\">".$sSearch."</a>");
	}
	/* parse results from a default Shodan search */
	function ParseDefaultShodan($aData){
		$iMatches = count($aData);
		for($x=0;$x<count($aData);$x++){
			foreach($aData[$x] as $sKey=>$mValue){
				if($mValue==""){
					unset($aData[$x][$sKey]);
				}
				elseif(is_array($mValue) && isset($mValue[0]) && $mValue[0]==""){
					unset($aData[$x][$sKey]);
				}
				elseif($sKey=="cve"){
					// create direct links if CVE is available
					for($y=0;$y<count($mValue);$y++){
						$aData[$x][$sKey] = $this->CVEID($mValue[$y]);
					}
				}
				elseif($sKey=="osvdb"){
					// create direct links if OSVDB is available
					for($y=0;$y<count($mValue);$y++){
						$aData[$x][$sKey] = $this->OSVDB($mValue[$y]);
					}
				}
				elseif($sKey=="bid"){
					// create direct links if BugTraq id is available
					for($y=0;$y<count($mValue);$y++){
						$aData[$x][$sKey] = $this->BugTraqID($mValue[$y]);
					}
				}
				elseif($sKey=="code"){
					$aData[$x][$sKey] = stripslashes($aData[$x][$sKey]);
					if(strlen($aData[$x][$sKey])>500){
						// assuming it's code, just put it in a textarea
						$aData[$x][$sKey] = "<textarea name=\"sCode\" rows=\"10\" cols=\"40\">".$aData[$x][$sKey]."</textarea>";
					}
					else{
						// just printing it as a string on the page
						$aData[$x][$sKey] = nl2br($aData[$x][$sKey]);
					}
				}
			}
		}
		return($aData);
	}
	/* parse results from a host search with(out) filters */
	function ParseHostNoFilters($aData){
		$iMatches = count($aData);
		$aResultSet = array();
		for($x=0;$x<$iMatches;$x++){
			$aResultSet[$x] = array();
			$aResultSet[$x]['ip'] = $aData[$x]['ip_str'];
			$aResultSet[$x]['product'] = $aData[$x]['product']!="" ? $this->Google($aData[$x]['product']) : "n/a";
			$aResultSet[$x]['version'] = $aData[$x]['version'];
			$aResultSet[$x]['devicetype'] = $aData[$x]['devicetype'];
			$aResultSet[$x]['port'] = $aData[$x]['port'];
			$aResultSet[$x]['asn'] = !isset($aData[$x]['asn']) ? "n/a" : $this->ASN($aData[$x]['asn']);
			$aResultSet[$x]['last_update'] = substr(str_replace("T"," ",$aData[$x]['timestamp']),0,strpos($aData[$x]['timestamp'],"."));
			$aResultSet[$x]['isp'] = $aData[$x]['isp'];
			$aResultSet[$x]['transport'] = $aData[$x]['transport'];
			$aResultSet[$x]['city'] = $aData[$x]['location']['city'];
			$aResultSet[$x]['country'] = $aData[$x]['location']['country_name'];
			$aResultSet[$x]['country_code'] = $aData[$x]['location']['country_code'];
			$aResultSet[$x]['dma_code'] = $aData[$x]['location']['dma_code']!="" ? $aData[$x]['location']['dma_code'] : "n/a";
			$aResultSet[$x]['area_code'] = $aData[$x]['location']['area_code']!="" ? $aData[$x]['location']['area_code'] : "n/a";
			$aResultSet[$x]['region_code'] = $aData[$x]['location']['region_code']!="" ? $aData[$x]['location']['region_code'] : "n/a";
			$aResultSet[$x]['postal_code'] = $aData[$x]['location']['postal_code'];
			$aResultSet[$x]['latitude'] = $aData[$x]['location']['latitude'];
			$aResultSet[$x]['longitude'] = $aData[$x]['location']['longitude'];
			$aResultSet[$x]['http_host'] = $aData[$x]['http']['host'];
			$aResultSet[$x]['http_server'] = $aData[$x]['http']['server'];
			$aResultSet[$x]['http_location'] = $aData[$x]['http']['location']!="" ? $aData[$x]['http']['location'] : "n/a";
			$aResultSet[$x]['organization'] = $aData[$x]['org']!="" ? $this->GoogleFeelingLucky($aData[$x]['org']) : "n/a";
			$aResultSet[$x]['os'] = $aData[$x]['os']!="" ? $aData[$x]['os'] : "n/a";
			$aResultSet[$x]['securitytxt'] = $aData[$x]['http']['securitytxt']!="" ? $aData[$x]['http']['securitytxt'] : "n/a";
			$aResultSet[$x]['securitytxthash'] = $aData[$x]['http']['securitytxt_hash']!="" ? $aData[$x]['http']['securitytxt_hash'] : "n/a";
			$aResultSet[$x]['components'] = isset($aData[$x]['http']['components'][0]) && $aData[$x]['http']['components'][0]!="" ? implode(" ,",$aData[$x]['http']['components']) : "";
			$aResultSet[$x]['hostnames'] = isset($aData[$x]['hostnames'][0]) && $aData[$x]['hostnames'][0]!="" ? implode(" ,",$aData[$x]['hostnames']) : "";
			$aResultSet[$x]['domains'] = isset($aData[$x]['domains'][0]) && $aData[$x]['domains'][0]!="" ? implode(" ,",$aData[$x]['domains']) : "";
			$aResultSet[$x]['data'] = "<textarea rows=\"10\" cols=\"60\">".$aData[$x]['data']."</textarea>";
			$aResultSet[$x]['html'] = "<textarea rows=\"10\" cols=\"60\">".$aData[$x]['http']['html']."</textarea>";
		}
		return($aResultSet);
	}
	/* parse results from an ip search */
	function ParseIPSearch($aData){
		$aResultSet = array();
		$aResultSet['ip'] = $aData[0]['ip_str'];
		$aResultSet['longip'] = $aData[0]['ip'];
		$aResultSet['country_code'] = $aData[0]['country_code'];
		$aResultSet['country_name'] = $aData[0]['country_name'];
		$aResultSet['city'] = $aData[0]['city'];
		$aResultSet['isp'] = $aData[0]['isp'];
		$aResultSet['postal_code'] = $aData[0]['postal_code']!="" ? $aData[0]['country_code'] : "n/a";
		$aResultSet['region_code'] = $aData[0]['region_code']!="" ? $aData[0]['region_code'] : "n/a";
		$aResultSet['area_code'] = $aData[0]['area_code']!="" ? $aData[0]['area_code'] : "n/a";
		$aResultSet['dma_code'] = $aData[0]['dma_code']!="" ? $aData[0]['dma_code'] : "n/a";
		$aResultSet['latitude'] = $aData[0]['latitude'];
		$aResultSet['longitude'] = $aData[0]['longitude'];
		$aResultSet['tags'] = !isset($aData[0]['tags'][0]) ? "n/a" : implode(", ",$aData[0]['tags']);
		$aResultSet['hostnames'] = !isset($aData[0]['hostnames'][0]) ? "n/a" : implode(", ",$aData[0]['hostnames']);
		$aResultSet['domains'] = !isset($aData[0]['data'][0]['domains'][0]) ? "n/a" : implode(", ",$aData[0]['data'][0]['domains']);
		$aResultSet['asn'] = !isset($aData[0]['data'][0]['asn']) ? "n/a" : $this->ASN($aData[0]['data'][0]['asn']);
		$aResultSet['http_redirects'] = !isset($aData[0]['data'][0]['http']['redirects'][0]) ? "n/a" : implode(", ",$aData[0]['data'][0]['http']['redirects']);
		$aResultSet['http_components'] = !isset($aData[0]['data'][0]['http']['components'][0]) ? "n/a" : implode(", ",$aData[0]['data'][0]['http']['components']);
		$aResultSet['ports'] = !isset($aData[0]['ports'][0]) ? "" : implode(", ",$aData[0]['ports']);
		$aResultSet['os'] = $aData[0]['os']!="" ? $aData[0]['data'][0]['os'] : "unknown";
		$aResultSet['transport'] = $aData[0]['data'][0]['transport'];
		$aResultSet['hash'] = $aData[0]['data'][0]['hash'];
		$aResultSet['organization'] = $aData[0]['org']!="" ? $this->GoogleFeelingLucky($aData[0]['org']) : "n/a";
		$aResultSet['product'] = $aData[0]['data'][0]['product']!="" ? $this->Google($aData[0]['data'][0]['product']) : "n/a";;
		$aResultSet['data'] = "<textarea rows=\"10\" cols=\"60\">".$aData[0]['data'][0]['data']."</textarea>";
		$aResultSet['http_host'] = $aData[0]['data'][0]['http']['host'];
		$aResultSet['http_html'] = "<textarea rows=\"10\" cols=\"60\">".$aData[0]['data'][0]['http']['html']."</textarea>";
		$aResultSet['http_location'] = $aData[0]['data'][0]['http']['location'];
		$aResultSet['last_update'] = substr(str_replace("T"," ",$aData[0]['data'][0]['timestamp']),0,strpos($aData[0]['data'][0]['timestamp'],"."));
		$aResultSet['http_title'] = $aData[0]['data'][0]['http']['title'];
		$aResultSet['http_server'] = $aData[0]['data'][0]['http']['server']!="" ? $aData[0]['data'][0]['http']['server'] : "unknown";
		$aResultSet['http_sitemap'] = $aData[0]['data'][0]['http']['sitemap']!="" ? $aData[0]['data'][0]['http']['sitemap'] : "n/a";
		$aResultSet['http_securitytxt'] = $aData[0]['data'][0]['http']['securitytxt']!="" ? $aData[0]['data'][0]['http']['securitytxt'] : "n/a";
		$aResultSet['http_securitytxt_hash'] = $aData[0]['data'][0]['http']['securitytxt_hash']!="" ? $aData[0]['data'][0]['http']['securitytxt_hash'] : "n/a";
		return($aResultSet);
	}
	/* generate a unique filename */
	function GenerateFileName(){
		return(time()."-".substr(md5(mt_rand(0,1000)),0,mt_rand(5,9)));
	}
	/* paginate search results */
	function Paginate($aResults,$iCurrentPage){
		if(!$iResults = @count($aResults)){
			return(false);
		}
		$iPages = ceil($iResults/$this->GetMatches());
		if(!@is_numeric($iCurrentPage) || $iCurrentPage<=0 || $iCurrentPage>$iPages){
			$iCurrentPage = 1;
		}
		return(array_slice($aResults,($iCurrentPage-1)*$iPages,$this->GetMatches()));
	}
	/* paginate search results into html navigation and add help link */
	function PaginateHTML($iResults,$iPage){
		$iPages = ceil($iResults/$this->GetMatches());
		$iPage = @intval($iPage);
		if(!is_numeric($iPage) || $iPage<=0 || $iPage>$iPages){
			// invalid value will be treated as 1, first page
			$iPage = 1;
		}
		$sBuffer = "<div class=\"navpag\">";
		if($iPage!=1){
			// base64_encode(settype:filename:page)
			// base64_encode(settype:search:page)
			$sBuffer .= "<a href=\"#\" class=\"wrapsrch\" alt=\"".$this->CreateBase64PostData($this->GetPage()-1)."\">previous</a>";
		}
		$sBuffer .= " ".$iPage." ";
		if($iPage!=$iPages){
			$sBuffer .= "<a href=\"#\" class=\"wrapsrch\" alt=\"".$this->CreateBase64PostData($this->GetPage()+1)."\">next</a>";
		}
		$sBuffer .= " - <a href=\"https://developer.shodan.io/api/banner-specification\" target=\"blank\">[help]</a></div>";
		return($sBuffer);
	}
	/* creates encoded data that will be used to send several variables through a hyperlink's alt property */
	function CreateBase64PostData($iPage){
		return(base64_encode($this->GetMethod().":".$this->GetSearch().":".$iPage));
	}
}
?>